@@ -2,8 +2,9 @@ package Module::CPANTS::Kwalitee::Pod;
 use warnings;
 use strict;
 use File::Spec::Functions qw/catfile/;
+use Encode;
 
-our $VERSION = '0.93_02';
+our $VERSION = '0.93_03';
 $VERSION = eval $VERSION; ## no critic
 
 our @ABSTRACT_STUBS = (
@@ -22,17 +23,22 @@ sub analyse {
     my ($class, $me) = @_;
     my $distdir = $me->distdir;
     my %abstract;
+    my @errors;
     for my $module (@{$me->d->{modules} || []}) {
-        my ($package, $abstract) = $class->_parse_abstract(catfile($distdir, $module->{file}));
+        my ($package, $abstract, $error) = $class->_parse_abstract(catfile($distdir, $module->{file}));
+        push @errors, "$error ($package)" if $error;
         $me->d->{abstracts_in_pod}{$package} = $abstract if $package;
     }
 
     # sometimes pod for .pm file is put into .pod
     for my $file (@{$me->d->{files_array} || []}) {
         next unless $file =~ /\.pod$/ && ($file =~ m!^lib/! or $file =~ m!^[^/]+$!);
-        my ($package, $abstract) = $class->_parse_abstract(catfile($distdir, $file));
+        local $@;
+        my ($package, $abstract, $error) = $class->_parse_abstract(catfile($distdir, $file));
+        push @errors, "$error ($package)" if $error;
         $me->d->{abstracts_in_pod}{$package} = $abstract if $package;
     }
+    $me->d->{error}{has_abstract_in_pod} = join ';', @errors if @errors;
 }
 
 # adapted from ExtUtils::MM_Unix and Module::Build::PodParser
@@ -42,7 +48,11 @@ sub _parse_abstract {
     my $inpod = 0;
     open my $fh, '<', $file or return;
     my $directive;
+    my $encoding;
     while(<$fh>) {
+        if (/^=encoding\s+(.+)/) {
+            $encoding = $1;
+        }
         if (/^=(?!cut)(.+)/) {
             $directive = $1;
             $inpod = 1;
@@ -61,7 +71,21 @@ sub _parse_abstract {
         s/\s+$//s;
         $abstract .= "\n$_";
     }
-    return ($package, $abstract);
+
+    my $error;
+    if ($encoding) {
+        my $encoder = find_encoding($encoding);
+        if (!$encoder) {
+            $error = "unknown encoding: $encoding";
+        } else {
+            $abstract = eval { $encoder->decode($abstract) };
+            if ($@) {
+                $error = $@;
+                $error =~ s|\s*at .+ line \d+.+$||s;
+            }
+        }
+    }
+    return ($package, $abstract, $error);
 }
 
 ##################################################################
@@ -76,6 +100,7 @@ sub kwalitee_indicators {
           remedy => q{Provide a short description in the NAME section of the pod (after the module name followed by a hyphen) at least for the main module of this distribution.},
           code => sub {
               my $d = shift;
+              return 0 if $d->{error}{has_abstract_in_pod};
               my @abstracts = grep {defined $_ && length $_} values %{$d->{abstracts_in_pod} || {}};
               return @abstracts ? 1 : 0;
           },
@@ -94,7 +119,7 @@ sub kwalitee_indicators {
               my %mapping = map {$_ => 1} @ABSTRACT_STUBS;
               my @errors;
               for (sort keys %{$d->{abstracts_in_pod} || {}}) {
-                  push @errors, $_ if $mapping{$d->{abstracts_in_pod}{$_}};
+                  push @errors, $_ if $mapping{$d->{abstracts_in_pod}{$_} || ''};
               }
               if (@errors) {
                   $d->{error}{no_abstract_stub_in_pod} = join ',', @errors;
